import greenfoot.*;  // (Actor, World, Greenfoot, GreenfootImage)

public class CrabWorld extends World
{
    
    private String version = "Super Crab v.0.96";
    
    //Lobster Placement Variables
    private int numberOfLobsters = 2;
    private int lobsterMoveDelay = 0; //Number of act calls to wait before lobster moves again. 0 is normal speed. Above 0 is slower.
    private int lobsterMoveRepeat = 1; //The number of times Lobster moves at once. 1 is normal speed.
    private int crabSafetyRange = 150; //The minimum distance away from Crab Lobsters can spawn
    
    //Crab Placement Variables
    private int crabX = 320;
    private int crabY = 240;
    
    //Worm Lifespan Variables
    private int wormLife = 100;
    private int wormDeathRate = 20;
    
    private String easyKey = "1";
    private String mediumKey = "2";
    private String hardKey = "3";
    private DifficultyChooser diffChooser = new DifficultyChooser(easyKey,mediumKey,hardKey);
    private boolean diffChosen = false;
    
    private Title splash = new Title();

    /**
     * Create the crab world (the beach). Our world has a size 
     * of 640x480 cells, where every cell is just 1 pixel.
     * Also adds the difficulty selection text, the super crab title and draw's the version string.
     */
    public CrabWorld() //This is the CrabWorld's Constructor it is run when the world is initialised
    {
        super(640, 480, 1); //This is how we call the constructor of the class we extended
        addObject(diffChooser, 320,320);
        addObject(splash, 320, 100);
        drawVersion();
    }
    
    /**
     * Checks key input for difficulty
     */
    public void act(){
        if(!diffChosen){
            if(Greenfoot.isKeyDown(easyKey)){
                setDifficulty(2, 1, 1, 200, 10);
            }
            if(Greenfoot.isKeyDown(mediumKey)){
                setDifficulty(3, 0, 1, 150, 10 );
            }
            if(Greenfoot.isKeyDown(hardKey)){
                setDifficulty(5, 0, 1, 100, 10);
            }
        }
    }
    
    /**
     * Populates the world with the assigned number and speed of lobster, as well as the first worm and the crab.
     */
    private void populate(){
        //Adds the first worm, and informs worm of it's lifespan and extiction speed
        addObject( new Worm(wormLife,wormDeathRate), Greenfoot.getRandomNumber(640), Greenfoot.getRandomNumber(480) );
        
        /*
         * Adds lobsters at random X and Y Positions, given that they are not within crabSafetyRange pixels of
         * the crab's spawn point
         */
        for(int i = 0; i < numberOfLobsters; i++){
            int testX = 0;
            int testY = 0;
            boolean xOK = false;
            boolean yOK = false;
            do {
                testX = Greenfoot.getRandomNumber(640);
                testY = Greenfoot.getRandomNumber(480);
                //System.out.println("X Might be: " + testX);
                xOK = (testX > crabX - crabSafetyRange) && (testX < crabX + crabSafetyRange);
                yOK = (testY > crabY - crabSafetyRange) && (testY < crabY + crabSafetyRange);
            } while (xOK && yOK);
            
            addObject( new Lobster(lobsterMoveDelay, lobsterMoveRepeat), testX, testY);
        }
        addObject( new Crab(), 320, 240);
    }
    
    /**
     * Draws the version string onto the world image. 
     */
    private void drawVersion(){
        GreenfootImage bg = getBackground();
        bg.setColor(new java.awt.Color(0,0,0));
        bg.drawString(version, 4, 12);
        setBackground(bg);
    }
    
    /**
     * Sets difficulty factors. Set's Lobster quantity and speed. Can not be carried out once game begins.
     * numLob - Set's number of Lobsters
     * lobDelay - 0 is normal speed. Any number above 1 is a slowdown.
     * lobRepeat - 1 is normal speed. 0 is no speed. Above 1 is faster.
     */
    private void setDifficulty(int numLob, int lobDelay, int lobRepeat, int wormL, int wormD){
        numberOfLobsters = numLob;
        lobsterMoveDelay = lobDelay;
        lobsterMoveRepeat = lobRepeat;
        wormLife = wormL;
        wormDeathRate = wormD;
        diffChosen = true;
        populate();
        removeObject(diffChooser);
        removeObject(splash);
    }
}